"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var OBJECT = exports.OBJECT = "object";

var isDevelopment = exports.isDevelopment = function isDevelopment() {
  return process.env.NODE_ENV !== "production";
};

var isFunction = function isFunction(f) {
  return typeof f === "function";
};

var toArray = exports.toArray = function toArray(field) {
  if (Array.isArray(field)) {
    return field;
  } else {
    return [field];
  }
};

var toError = exports.toError = function toError(message) {
  if (isDevelopment()) {
    throw new ReferenceError(message);
  } else {
    console.error(message);
  }
};

var hasField = function hasField(field, schema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    return schema.properties[field] !== undefined;
  } else {
    var parentField = field.substr(0, separator);
    var refSch = toRefSchema(parentField, schema);
    if (refSch) {
      var refSchema = fetchRefSchema(refSch, schema);
      return refSchema ? hasField(field.substr(separator + 1), refSchema) : false;
    } else {
      toError("Failed to find " + refSch + " for " + field);
      return false;
    }
  }
};

var validateFields = exports.validateFields = function validateFields(action, fetchFields) {
  if (!fetchFields) {
    toError("validateFields requires fetchFields function");
    return;
  }
  return function (params, schema) {
    var relFields = isFunction(fetchFields) ? toArray(fetchFields(params)) : toArray(fetchFields);
    relFields.filter(function (field) {
      return !hasField(field, schema);
    }).forEach(function (field) {
      return toError("Field \"" + field + "\" is missing from schema on \"" + action + "\"");
    });
  };
};

var fetchRefSchema = function fetchRefSchema(ref, schema) {
  if (ref.startsWith("#/")) {
    ref.substr(2).split("/");
    return ref.substr(2).split("/").reduce(function (schema, field) {
      return schema[field];
    }, schema);
  } else if (schema.properties[ref] && schema.properties[ref].type === OBJECT) {
    return schema.properties[ref];
  } else {
    toError("Only local references supported at this point");
    return undefined;
  }
};

var toRefSchema = function toRefSchema(field, _ref) {
  var properties = _ref.properties;

  if (properties[field]) {
    if (properties[field]["$ref"]) {
      return properties[field]["$ref"];
    } else if (properties[field].items && properties[field].items["$ref"]) {
      return properties[field].items["$ref"];
    } else if (properties[field].type === OBJECT) {
      return field;
    }
  }
  return undefined;
};

/**
 * Find relevant schema for the field
 * @returns { field: "string", schema: "object" } relevant field and schema
 */
var findRelSchemaAndField = exports.findRelSchemaAndField = function findRelSchemaAndField(field, schema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    return { field: field, schema: schema };
  }

  var parentField = field.substr(0, separator);
  var refSch = toRefSchema(parentField, schema);
  if (!refSch) {
    toError("Failed to find " + refSch);
    return { field: field, schema: schema };
  }

  var refSchema = fetchRefSchema(refSch, schema);
  if (refSchema) {
    return findRelSchemaAndField(field.substr(separator + 1), refSchema);
  }

  toError("Failed to retrieve " + refSchema + " from schema");
  return { field: field, schema: schema };
};